import React from "react";
import { AppProviderProps, defaultAppProviderProps } from "../types";

const AppContext = React.createContext<AppProviderProps>(
  defaultAppProviderProps
);

export function AppProvider(props: AppProviderProps) {
  const [type] = React.useState(props.type);
  const [tokenStorageKey] = React.useState(props.tokenStorageKey);
  const [token, setToken] = React.useState<string | null>(props.token);

  React.useEffect(() => {
    if (typeof tokenStorageKey === "string" && tokenStorageKey !== "") {
      const jwt = localStorage.getItem(tokenStorageKey);
      setToken(jwt);
    }
  }, [tokenStorageKey]);

  React.useEffect(() => {
    if (typeof tokenStorageKey === "string" && tokenStorageKey !== "") {
      if (typeof token === "string" && token !== "") {
        localStorage.setItem(tokenStorageKey, token);
      } else if (localStorage.getItem(tokenStorageKey)) {
        localStorage.removeItem(tokenStorageKey);
      }
    }
  }, [tokenStorageKey, token]);

  const memData = React.useMemo(() => {
    return { type, token, tokenStorageKey: undefined, setToken };
  }, [type, token, tokenStorageKey, setToken]);

  return <AppContext.Provider value={memData} {...props} />;
}

export function useApp() {
  const context = React.useContext(AppContext);
  if (!context) {
    // eslint-disable-next-line no-throw-literal
    throw "error: app context not defined.";
  }
  return context;
}
