import React from "react";
import { emptyRequest, getRequest, postRequest } from "../utils";
import { useHttp } from "./useHttp";
import { useApp } from "./useApp";
import { AuthProviderProps, defaultAuthProviderProps } from "../types";

const AuthContext = React.createContext<AuthProviderProps>(
  defaultAuthProviderProps
);

const empty = emptyRequest();

export function AuthProvider(props: AuthProviderProps) {
  const { token, setToken } = useApp();
  const [sessionRequest, setSessionRequest] = React.useState(empty);
  const [session, sessionLoading] = useHttp(sessionRequest);
  const [agendaRequest, setAgendaRequest] = React.useState(empty);
  const [agendaResponse, agendaResponseLoading] = useHttp(agendaRequest);

  React.useEffect(() => {
    if (session && !sessionLoading) {
      if (session?.resultado?.[0]) {
        const { token } = session.resultado[0];
        setToken(token);
      }
    }
  }, [session, sessionLoading, setToken]);

  React.useEffect(() => {
    if (token) {
      const agendaReq = getRequest("agenda/cuenta", {
        expand: "permiso,agenda,consejoElectoral,consejosExtra,usuario",
      });
      // agendaReq.alert = true;
      setAgendaRequest(() => agendaReq);
    } else {
      setAgendaRequest(empty);
    }
  }, [token]);

  const signIn = React.useCallback(async (email, password) => {
    try {
      if (email !== "" && password !== "") {
        const req = postRequest("login", { correo: email, clave: password });
        setSessionRequest({ ...req });
      }
    } catch (e) {
      console.log(e);
    }
  }, []);

  const signOut = React.useCallback(async () => {
    try {
      setToken(null);
      setSessionRequest(empty);
    } catch (e) {
      console.log(e);
    }
  }, [setToken]);

  const memData = React.useMemo(() => {
    return {
      session,
      sessionLoading,
      user:
        agendaResponse &&
        agendaResponse.resultado &&
        agendaResponse.resultado.length > 0
          ? {
              ...agendaResponse.resultado[0],
              permisoExtra: agendaResponse.detalle,
            }
          : null,
      userLoading: agendaResponseLoading,
      signIn,
      signOut,
    };
  }, [
    session,
    sessionLoading,
    agendaResponse,
    agendaResponseLoading,
    signIn,
    signOut,
  ]);

  return <AuthContext.Provider value={memData} {...props} />;
}

export function useAuth() {
  const context = React.useContext(AuthContext);
  if (!context) {
    // eslint-disable-next-line no-throw-literal
    throw "error: auth context not defined.";
  }
  return context;
}
